"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompletableObservable = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const CompletionContext_1 = require("./CompletionContext");
const CompletionError_1 = require("./CompletionError");
class CompletableObservable {
    observable;
    constructor(observable, hasContext = false) {
        if (hasContext)
            this.observable = observable;
        else
            this.observable = this.initializeObservable(observable);
    }
    thenApply(func) {
        return new CompletableObservable(this
            .observable
            .pipe((0, operators_1.map)(context => this.doApply(context, func))), true);
    }
    thenCompose(func) {
        return new CompletableObservable(this
            .observable
            .pipe((0, operators_1.mergeMap)(context => this.toObservable(this.doCompose(func, context))
            .pipe((0, operators_1.map)(context => context)))), true);
    }
    thenAccept(consumer, handler) {
        return new CompletableObservable(this
            .observable
            .pipe((0, operators_1.map)(context => this.doAccept(context, consumer, handler))), true);
    }
    exceptionally(func) {
        return new CompletableObservable(this
            .observable
            .pipe((0, operators_1.map)(context => this.doExceptionally(context, func))), true);
    }
    subscribe(consumer, handler) {
        if (consumer == null)
            consumer = result => { };
        if (handler == null)
            handler = error => { };
        this
            .observable
            .subscribe(context => {
            if (context.hasError())
                handler(context.getError());
            else
                consumer(context.getResult());
        }, error => handler(error));
    }
    static fromResult(result) {
        return new CompletableObservable((0, rxjs_1.of)(result), false);
    }
    static fromObservable(observable) {
        return new CompletableObservable(observable, false);
    }
    static fromPromise(promise) {
        return new CompletableObservable((0, rxjs_1.from)(promise), false);
    }
    static fromError(error) {
        return new CompletableObservable(new rxjs_1.Observable(observer => observer.next(CompletionContext_1.CompletionContext.ofError(new CompletionError_1.CompletionError(error)))), true);
    }
    static supplyAsync(supplier) {
        return new CompletableObservable(new rxjs_1.Observable(observer => observer.next(supplier())), false);
    }
    initializeObservable(input) {
        return input.pipe((0, operators_1.map)(value => CompletionContext_1.CompletionContext.ofResult(value)));
    }
    doApply(context, func) {
        console.log("doApply");
        if (context.hasError())
            return CompletionContext_1.CompletionContext.ofError(context.getError());
        try {
            return CompletionContext_1.CompletionContext.ofResult(func(context.getResult()));
        }
        catch (e) {
            console.log("Caught exception in doApply");
            return CompletionContext_1.CompletionContext.ofError(new CompletionError_1.CompletionError(e));
        }
    }
    doCompose(func, context) {
        console.log("doCompose");
        if (context.hasError())
            return CompletionContext_1.CompletionContext.ofError(context.getError());
        try {
            return CompletionContext_1.CompletionContext.ofResult(func(context.getResult()));
        }
        catch (e) {
            console.log("Caught exception in doCompose");
            return CompletionContext_1.CompletionContext.ofError(e);
        }
    }
    doAccept(context, consumer, handler) {
        console.log("doAccept");
        if (context.hasError()) {
            if (handler != null) {
                try {
                    handler(context.getError());
                }
                catch (e) {
                    console.log("Caught exception in doAccept");
                    return context.setError(new CompletionError_1.CompletionError(e));
                }
            }
        }
        else {
            try {
                consumer(context.getResult());
            }
            catch (e) {
                return context.setError(new CompletionError_1.CompletionError(e));
            }
        }
        return context;
    }
    doExceptionally(context, func) {
        console.log("doExceptionally");
        if (context.hasError())
            return CompletionContext_1.CompletionContext.ofResult(func(context.getError()));
        return context;
    }
    toObservable(input) {
        if (input.hasResult())
            return input.getResult().observable;
        return this.observable.pipe((0, operators_1.map)((context, ignore) => CompletionContext_1.CompletionContext.ofError(input.getError())));
    }
}
exports.CompletableObservable = CompletableObservable;
//# sourceMappingURL=CompletableObservable.js.map